@@ -1,21 +1,24 @@
 package Email::Sender::Transport::SMTP::TLS;
 
-use Moose;
-with 'Email::Sender::Transport';
+BEGIN {
+    $Email::Sender::Transport::SMTP::TLS::VERSION = '0.05';
+}
 
-our $VERSION = '0.03';
+# ABSTRACT: Email::Sender with L<Net::SMTP::TLS> (Eg. Gmail)
+
+use Moose 0.90;
 
 use Net::SMTP::TLS;
 use Email::Sender::Failure::Multi;
 use Email::Sender::Success::Partial;
 use Email::Sender::Util;
 
-has host => (is => 'ro', isa => 'Str', default => 'localhost');
-has port => (is => 'ro', isa => 'Int', default => 587 );
-has username => (is => 'ro', isa => 'Str', required => 1);
-has password => (is => 'ro', isa => 'Str', required => 1);
-has allow_partial_success => (is => 'ro', isa => 'Bool', default => 0);
-has helo      => (is => 'ro', isa => 'Str'); # default to hostname_long
+has host     => ( is => 'ro', isa => 'Str', default  => 'localhost' );
+has port     => ( is => 'ro', isa => 'Int', default  => 587 );
+has username => ( is => 'ro', isa => 'Str', required => 1 );
+has password => ( is => 'ro', isa => 'Str', required => 1 );
+has allow_partial_success => ( is => 'ro', isa => 'Bool', default => 0 );
+has helo => ( is => 'ro', isa => 'Str' );    # default to hostname_long
 
 # From http://search.cpan.org/src/RJBS/Email-Sender-0.000/lib/Email/Sender/Transport/SMTP.pm
 ## I am basically -sure- that this is wrong, but sending hundreds of millions of
@@ -39,13 +42,14 @@ sub _smtp_client {
     eval {
         $smtp = Net::SMTP::TLS->new(
             $self->host,
-            Port => $self->port,
-            User => $self->username,
+            Port     => $self->port,
+            User     => $self->username,
             Password => $self->password,
-            $self->helo ? (Hello => $self->helo) : (),
+            $self->helo ? ( Hello => $self->helo ) : (),
         );
     };
 
+    warn $@ if $@;
     $self->_throw($@) if $@;
     $self->_throw("unable to establish SMTP connection") unless $smtp;
 
@@ -53,52 +57,47 @@ sub _smtp_client {
 }
 
 sub _throw {
-    my ($self, @rest) = @_;
+    my ( $self, @rest ) = @_;
     Email::Sender::Util->_failure(@rest)->throw;
 }
 
 sub send_email {
-    my ($self, $email, $env) = @_;
+    my ( $self, $email, $env ) = @_;
 
     Email::Sender::Failure->throw("no valid addresses in recipient list")
-        unless my @to = grep { defined and length } @{ $env->{to} };
+      unless my @to = grep { defined and length } @{ $env->{to} };
 
     my $smtp = $self->_smtp_client;
 
-    my $FAULT = sub { $self->_throw($_[0], $smtp); };
+    my $FAULT = sub { $self->_throw( $_[0], $smtp ); };
 
-    eval {
-        $smtp->mail(_quoteaddr($env->{from}));
-    };
+    eval { $smtp->mail( _quoteaddr( $env->{from} ) ); };
     $FAULT->("$env->{from} failed after MAIL FROM: $@") if $@;
 
     my @failures;
     my @ok_rcpts;
-  
+
     for my $addr (@to) {
-        eval {
-            $smtp->to(_quoteaddr($addr));
-        };
-        unless ( $@ ) {
+        eval { $smtp->to( _quoteaddr($addr) ); };
+        unless ($@) {
             push @ok_rcpts, $addr;
-        } else {
+        }
+        else {
+
             # my ($self, $error, $smtp, $error_class, @rest) = @_;
-            push @failures, Email::Sender::Util->_failure(
-                undef,
-                $smtp,
-                recipients => [ $addr ],
-            );
+            push @failures,
+              Email::Sender::Util->_failure( undef, $smtp,
+                recipients => [$addr], );
         }
     }
 
-    if (
-        @failures
-        and ((@ok_rcpts == 0) or (! $self->allow_partial_success))
-    ) {
+    if ( @failures
+        and ( ( @ok_rcpts == 0 ) or ( !$self->allow_partial_success ) ) )
+    {
         $failures[0]->throw if @failures == 1;
 
         my $message = sprintf '%s recipients were rejected during RCPT',
-            @ok_rcpts ? 'some' : 'all';
+          @ok_rcpts ? 'some' : 'all';
 
         Email::Sender::Failure::Multi->throw(
             message  => $message,
@@ -106,37 +105,78 @@ sub send_email {
         );
     }
 
+    my $message;
     eval {
         $smtp->data();
         $smtp->datasend( $email->as_string );
         $smtp->dataend;
+        $message = $smtp->message;
         $smtp->quit;
     };
+
     # ignore $@
 
-  # XXX: We must report partial success (failures) if applicable.
-    return $self->success unless @failures;
-    return Email::Sender::Success::Partial->new({
-        failure => Email::Sender::Failure::Multi->new({
-          message  => 'some recipients were rejected during RCPT',
-          failures => \@failures
-        }),
-    });
+    # XXX: We must report partial success (failures) if applicable.
+    return $self->success( { message => $message } ) unless @failures;
+    return $self->partial_success(
+        {
+            message => $message,
+            failure => Email::Sender::Failure::Multi->new(
+                {
+                    message  => 'some recipients were rejected during RCPT',
+                    failures => \@failures
+                }
+            ),
+        }
+    );
 }
 
+my %SUCCESS_CLASS;
+
+BEGIN {
+    $SUCCESS_CLASS{FULL} = Moose::Meta::Class->create_anon_class(
+        superclasses => ['Email::Sender::Success'],
+        roles        => ['Email::Sender::Role::HasMessage'],
+        cache        => 1,
+    );
+    $SUCCESS_CLASS{PARTIAL} = Moose::Meta::Class->create_anon_class(
+        superclasses => ['Email::Sender::Success::Partial'],
+        roles        => ['Email::Sender::Role::HasMessage'],
+        cache        => 1,
+    );
+}
+
+sub success {
+    my $self    = shift;
+    my $success = $SUCCESS_CLASS{FULL}->name->new(@_);
+}
+
+sub partial_success {
+    my ( $self, @args ) = @_;
+    my $obj = $SUCCESS_CLASS{PARTIAL}->name->new(@args);
+    return $obj;
+}
+
+with 'Email::Sender::Transport';
 __PACKAGE__->meta->make_immutable;
 no Moose;
 1;
 
-__END__
+=pod
 
 =head1 NAME
 
 Email::Sender::Transport::SMTP::TLS - Email::Sender with L<Net::SMTP::TLS> (Eg. Gmail)
 
+=head1 VERSION
+
+version 0.05
+
 =head1 SYNOPSIS
 
+    use Email::Sender::Simple qw(sendmail);
     use Email::Sender::Transport::SMTP::TLS;
+    use Try::Tiny;
 
     my $sender = Email::Sender::Transport::SMTP::TLS->new(
         host => 'smtp.gmail.com',
@@ -146,7 +186,7 @@ Email::Sender::Transport::SMTP::TLS - Email::Sender with L<Net::SMTP::TLS> (Eg.
         helo => 'fayland.org',
     );
     
-    #   my $message = Mail::Message->read($rfc822)
+    # my $message = Mail::Message->read($rfc822)
     #         || Email::Simple->new($rfc822)
     #         || Mail::Internet->new([split /\n/, $rfc822])
     #         || ...
@@ -163,21 +203,19 @@ Email::Sender::Transport::SMTP::TLS - Email::Sender with L<Net::SMTP::TLS> (Eg.
         body => 'Content.',
     );
     
-    eval {
-        $sender->send($message, {
-            from => 'username@gmail.com',
-            to   => [ 'to@mail.com' ],
-        } );
+    try {
+        sendmail($message, { transport => $transport });
+    } catch {
+        die "Error sending email: $_";
     };
-    die "Error sending email: $@" if $@;
 
 =head1 DESCRIPTION
 
 L<Email::Sender> replaces the old and sometimes problematic L<Email::Send> library, while this module replaces the L<Email::Send::SMTP::TLS>.
 
-It's still alpha. use it at your own risk!
+It is still alpha, but it works. use it at your own risk!
 
-=head1 ATTRIBUTES
+=head2 ATTRIBUTES
 
 The following attributes may be passed to the constructor:
 
@@ -197,7 +235,7 @@ The following attributes may be passed to the constructor:
 
 =back
 
-=head1 PARTIAL SUCCESS
+=head2 PARTIAL SUCCESS
 
 If C<allow_partial_success> was set when creating the transport, the transport
 may return L<Email::Sender::Success::Partial> objects.  Consult that module's
@@ -205,17 +243,15 @@ documentation.
 
 =head1 AUTHOR
 
-Fayland Lam, C<< <fayland at gmail.com> >>
+Fayland Lam <fayland@gmail.com>
 
-=head1 ACKNOWLEDGEMENTS
+=head1 COPYRIGHT AND LICENSE
 
-Ricardo SIGNES - L<Email::Sender>
+This software is copyright (c) 2010 by Fayland Lam.
 
-=head1 COPYRIGHT & LICENSE
-
-Copyright 2008 Fayland Lam, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+__END__
